<?php

namespace Drupal\spammaster;

use Drupal\Core\Database\Connection;
use Drupal\Core\Messenger\MessengerInterface;
use Symfony\Component\HttpFoundation\RequestStack;
use Drupal\Core\Session\AccountInterface;
use Drupal\Core\Session\AccountProxy;
use GuzzleHttp\Client;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\State\StateInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\Core\StringTranslation\TranslationInterface;
use Drupal\Component\Serialization\Json;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Class service.
 */
class SpamMasterElusiveService {
  use StringTranslationTrait;

  /**
   * The database connection object.
   *
   * @var \Drupal\Core\Database\Connection
   */
  protected $connection;

  /**
   * The Messenger service.
   *
   * @var \Drupal\Core\Messenger\MessengerInterface
   */
  protected $messenger;

  /**
   * The session account interface.
   *
   * @var \Drupal\Core\Session\AccountInterface
   */
  protected $account;

  /**
   * Symfony\Component\HttpFoundation\RequestStack definition.
   *
   * @var \Symfony\Component\HttpFoundation\RequestStack
   */
  protected $requestStack;

  /**
   * Client.
   *
   * @var \GuzzleHttp\Client
   */
  protected $httpClient;

  /**
   * The Config Factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * The state.
   *
   * @var \Drupal\Core\State\StateInterface
   */
  protected $state;

  /**
   * The String Translation..
   *
   * @var \Drupal\Core\StringTranslation\TranslationInterface
   */
  protected $stringTranslation;

  /**
   * Drupal\Core\Session\AccountProxy definition.
   *
   * @var \Drupal\Core\Session\AccountProxy
   */
  protected $currentUser;

  /**
   * The SpamMasterElusiveService service.
   *
   * @var \Drupal\spammmaster\SpamMasterElusiveService
   */
  protected $spamelusive;

  /**
   * The SpamMasterCollectService Service.
   *
   * @var \Drupal\spammaster\SpamMasterCollectService
   */
  protected $collectService;

  /**
   * The SpamMasterUserService Service.
   *
   * @var \Drupal\spammaster\SpamMasterUserService
   */
  protected $userService;

  /**
   * The form class.
   *
   * @var \Drupal\Core\Form\FormInterface
   */
  protected $form;

  /**
   * The form class.
   *
   * @var \Drupal\Core\Form\FormInterface
   */
  protected $formstate;

  /**
   * The form class.
   *
   * @var \Drupal\Core\Form\FormInterface
   */
  protected $spamFormId;

  /**
   * {@inheritdoc}
   */
  public function __construct(Connection $connection, MessengerInterface $messenger, AccountInterface $account, RequestStack $requestStack, Client $httpClient, ConfigFactoryInterface $configFactory, StateInterface $state, TranslationInterface $stringTranslation, AccountProxy $current_user, SpamMasterCollectService $collectService, SpamMasterUserService $userService) {
    $this->connection = $connection;
    $this->messenger = $messenger;
    $this->account = $account;
    $this->requestStack = $requestStack;
    $this->httpClient = $httpClient;
    $this->configFactory = $configFactory;
    $this->state = $state;
    $this->stringTranslation = $stringTranslation;
    $this->currentUser = $current_user;
    $this->collectService = $collectService;
    $this->userService = $userService;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('database'),
      $container->get('messenger'),
      $container->get('current_user'),
      $container->get('request_stack'),
      $container->get('http_client'),
      $container->get('config.factory'),
      $container->get('state'),
      $container->get('string_translation'),
      $container->get('current_user'),
      $container->get('spammaster.collect_service'),
      $container->get('spammaster.user_service')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function spamMasterElusiveCheck($spamelusive) {

    $this->spamelusive = $spamelusive;
    $spammaster_date = date("Y-m-d H:i:s");
    $spammaster_settings = $this->configFactory->getEditable('spammaster.settings');
    $spammaster_license = $spammaster_settings->get('spammaster.license_key');
    $spammaster_status = $this->state->get('spammaster.license_status');
    $spammaster_total_block_count = $this->state->get('spammaster.total_block_count');
    if ('VALID' === $spammaster_status || 'MALFUNCTION_1' === $spammaster_status || 'MALFUNCTION_2' === $spammaster_status) {

      // Call collect service.
      $collectnow = $this->requestStack->getCurrentRequest()->getClientIp();
      $spammaster_collect_service = $this->collectService;
      $is_collected = $spammaster_collect_service->spamMasterGetCollect($collectnow);
      $spamcollection = Json::decode($is_collected);
      $spammasterip = $spamcollection['spammasterip'];
      $spammasteragent = $spamcollection['spammasteragent'];
      $spammasterreferer = $spamcollection['spammasterreferer'];
      $spammasterurl = $spamcollection['spammasterurl'];

      // User service call.
      $spamuser = $this->currentUser->id();
      $spammaster_user_service = $this->userService;
      $is_user = $spammaster_user_service->spamMasterGetUser($spamuser);
      $spamuserdata = Json::decode($is_user);
      $spammasteruserid = $spamuserdata['ID'];
      $spammasterusername = $spamuserdata['username'];

      // Exempt whitelist.
      $spammaster_white_query = $this->connection->select('spammaster_white', 'u');
      $spammaster_white_query->fields('u', ['white']);
      $spammaster_white_query->where('(white = :ip)', [
        ':ip' => $spammasterip,
      ]);
      $spammaster_white_result = $spammaster_white_query->execute()->fetchObject();
      // White positive, or administrator log insert.
      $user_roles = $this->account->getRoles();
      if (!empty($spammaster_white_result) || in_array('administrator', $user_roles)) {
        $this->connection->insert('spammaster_keys')->fields([
          'date' => $spammaster_date,
          'spamkey' => 'spammaster-firewall',
          'spamvalue' => 'Spam Master: firewall within whitelist or registered user, Ip: ' . $spammasterip . ', Agent: ' . $spammasteragent . ', User ID: ' . $spammasteruserid . ', Username: ' . $spammasterusername,
        ])->execute();
        return 'BAIL';
      }
      else {
        // Buffer db check.
        $spammaster_spam_buffer_query = $this->connection->select('spammaster_threats', 'u');
        $spammaster_spam_buffer_query->fields('u', ['threat']);
        $spammaster_spam_buffer_query->where('(threat = :ip)', [
          ':ip' => $spammasterip,
        ]);
        $spammaster_spam_buffer_result = $spammaster_spam_buffer_query->execute()->fetchObject();
        // Buffer db positive, throw error, log insert.
        if (!empty($spammaster_spam_buffer_result)) {
          $spammaster_total_block_count_1 = ++$spammaster_total_block_count;
          $this->state->set('spammaster.total_block_count', $spammaster_total_block_count_1);

          $this->connection->insert('spammaster_keys')->fields([
            'date' => $spammaster_date,
            'spamkey' => 'spammaster-firewall',
            'spamvalue' => 'Spam Master: firewall BUFFER BLOCK, Ip: ' . $spammasterip . ', Agent: ' . $spammasteragent . ', User ID: ' . $spammasteruserid . ', Username: ' . $spammasterusername,
          ])->execute();
          return 'ELUSIVE';
        }
        else {
          if (!empty($spamelusive)) {
            // Exclusions by key.
            $spam_master_key_array = [
              'ajax_page_state',
              'Continue_Official_Verification',
              'Continue_Verification_Express',
              'delete-order-item',
              'edit_quantity',
              'editor_object',
              'Passer_a_la_Verification_Express',
              'Passer_a_la_Verification_Officielle',
              'purchased_entity',
              'search',
            ];
            // Exclusions by value.
            $spam_master_value_array = [
              'Add+to+cart',
              'Add to cart',
              'Add to wishlist',
              'Aggiungi al carrello',
              'Ajouter au panier',
              'angebot',
              'color_calc_form',
              'commerce_wishlist',
              'courses-',
              'Pay+and+complete+purchase',
              'Pay and complete purchase',
              'Proceed to checkout',
              'puppy_calc_form',
              'rabbit_calc_form',
              'rate',
              'search',
              'simplenews_subscri',
              'tfa_entry_form',
              'views_',
            ];
            foreach ($spamelusive as $spam_key => $spam_val) {
              // Exclusions by key loop and form_id loop.
              if (str_replace($spam_master_key_array, '', $spam_key) !== $spam_key) {
                $this->connection->insert('spammaster_keys')->fields([
                  'date' => $spammaster_date,
                  'spamkey' => 'spammaster-firewall',
                  'spamvalue' => 'Spam Master: submission key within post exclusion, Ip: ' . $spammasterip . ', Agent: ' . $spammasteragent,
                ])->execute();
                return 'BAIL';
              }
              if (!is_array($spam_val) && str_replace($spam_master_value_array, '', $spam_val) !== $spam_val) {
                $this->connection->insert('spammaster_keys')->fields([
                  'date' => $spammaster_date,
                  'spamkey' => 'spammaster-firewall',
                  'spamvalue' => 'Spam Master: submission value within post exclusion, Ip: ' . $spammasterip . ', Agent: ' . $spammasteragent,
                ])->execute();
                return 'BAIL';
              }
            }
          }

          // Exclude malformed or internal links.
          if (!empty($spammasterurl)) {
            if (stripos($spammasterurl, 'autodiscover.xml') !== FALSE) {
              return 'BAIL';
            }
          }

          // phpcs:ignore DrupalPractice.Variables.GetRequestData.SuperglobalAccessed
          if (!empty($_POST)) {
            // Web api check. Create data to be posted.
            $blog_threat_type = 'HAF';
            // phpcs:ignore DrupalPractice.Variables.GetRequestData.SuperglobalAccessed
            $result_post_content_json  = Json::encode($_POST);
            $result_post_content_trim  = substr($result_post_content_json, 0, 963);
            $result_post_content_clean = strip_tags($result_post_content_trim);
            if (empty($result_post_content_clean)) {
              $result_post_content_clean = 'is_elusive_d';
            }
            // Collect email to scan.
            if (!empty($spamelusive) && is_array($spamelusive)) {
              $match_string = implode(' ', $params);
              $match_string = preg_match('/[_a-z0-9-]+(\.[_a-z0-9-]+)*@[a-z0-9-]+(\.[a-z0-9-]+)*(\.[a-z]{2,3})/i', $match_string, $matches);
              if ($matches) {
                foreach ($matches as $key => $val) {
                  $resultKey = $key;
                  if ($resultKey) {
                  }
                  if (filter_var($val, FILTER_VALIDATE_EMAIL)) {
                    $spammasteremail = substr($val, 0, 256);
                    break;
                  }
                  else {
                    $spammasteremail = 'haf@' . rand(10000000, 99999999) . '.drup';
                  }
                }
              }
              else {
                $spammasteremail = 'haf@' . rand(10000000, 99999999) . '.drup';
              }
            }
            else {
              $spammasteremail = 'haf@' . rand(10000000, 99999999) . '.drup';
            }
            $blog_web_address = $this->requestStack->getCurrentRequest()->getHost();
            $address_unclean = $blog_web_address;
            $address = preg_replace('#^https?://#', '', $address_unclean);
            @$blog_server_ip = $_SERVER['SERVER_ADDR'];
            // If empty ip.
            if (empty($blog_server_ip) || $blog_server_ip == '0') {
              @$blog_server_ip = 'I ' . gethostbyname($_SERVER['SERVER_NAME']);
            }
            $spam_master_leaning_url = 'https://www.spammaster.org/core/learn/get_learn_haf.php';

            // Call drupal hhtpclient.
            $client = $this->httpClient;
            // Post data.
            $request = $client->post($spam_master_leaning_url, [
              'form_params' => [
                'blog_license_key'    => $spammaster_license,
                'blog_threat_ip'      => $spammasterip,
                'blog_threat_user'    => $is_user,
                'blog_threat_type'    => $blog_threat_type,
                'blog_threat_email'   => $spammasteremail,
                'blog_threat_content' => 'is_elusive_d - ' . $result_post_content_clean,
                'blog_threat_agent'   => $spammasteragent,
                'blog_threat_refe'    => $spammasterreferer,
                'blog_threat_dest'    => $spammasterurl,
                'blog_web_adress'     => $address,
                'blog_server_ip'      => $blog_server_ip,
              ],
            ]);
            // Decode json data or $request->getBody()->getContents().
            $response = Json::decode($request->getBody());
            if (empty($response['threat'])) {
              // Log clean rbl check.
              $this->connection->insert('spammaster_keys')->fields([
                'date' => $spammaster_date,
                'spamkey' => 'spammaster-firewall',
                'spamvalue' => 'Spam Master: firewall RBL CHECK, Ip: ' . $spammasterip . ', Agent: ' . $spammasteragent . ', User ID: ' . $spammasteruserid . ', Username: ' . $spammasterusername,
              ])->execute();
              return 'BAIL';
            }
            else {
              // Insert ip into buffer db.
              $this->connection->insert('spammaster_threats')->fields([
                'date' => $spammaster_date,
                'threat' => $spammasterip,
              ])->execute();
              // Web positive, return.
              $spammaster_total_block_count_1 = ++$spammaster_total_block_count;
              $this->state->set('spammaster.total_block_count', $spammaster_total_block_count_1);
              $this->connection->insert('spammaster_keys')->fields([
                'date' => $spammaster_date,
                'spamkey' => 'spammaster-firewall',
                'spamvalue' => 'Spam Master: firewall RBL BLOCK, Ip: ' . $spammasterip . ', Agent: ' . $spammasteragent . ', User ID: ' . $spammasteruserid . ', Username: ' . $spammasterusername,
              ])->execute();
              return 'ELUSIVE';
            }
          }
          else {
            return 'BAIL';
          }
        }
      }
    }
    else {
      return 'BAIL';
    }
  }

  /**
   * {@inheritdoc}
   */
  public function spamMasterIsJson($spamelusive) {

    $this->spamelusive = $spamelusive;
    $spammaster_status = $this->state->get('spammaster.license_status');
    $spammaster_total_block_count = $this->state->get('spammaster.total_block_count');
    if ('VALID' === $spammaster_status || 'MALFUNCTION_1' === $spammaster_status || 'MALFUNCTION_2' === $spammaster_status) {

      if (is_string($spamelusive) && is_array(Json::decode($spamelusive)) && (json_last_error() === JSON_ERROR_NONE)) {
        $spammaster_total_block_count_1 = ++$spammaster_total_block_count;
        $this->state->set('spammaster.total_block_count', $spammaster_total_block_count_1);

        // Decode json.
        $spam_elusive_decoded = Json::decode($spamelusive);
        foreach ($spam_elusive_decoded as $val) {
          if (preg_grep('/[_a-z0-9-]+(\.[_a-z0-9-]+)*@[a-z0-9-]+(\.[a-z0-9-]+)*(\.[a-z]{2,3})/i', $val)) {
            $matches = preg_grep('/^[_a-z0-9-]+(\.[_a-z0-9-]+)*@[a-z0-9-]+(\.[a-z0-9-]+)*(\.[a-z]{2,3})$/', $val);
            if ($matches) {
              foreach ($matches as $val) {
                // Return $val inside loop to spam master api.
                return 'BAIL';
              }
            }
            else {
              return 'BAIL';
            }
          }
          else {
            return 'BAIL';
          }
        }
      }
      else {
        return 'ELUSIVE';
      }
    }
    else {
      return 'BAIL';
    }
  }

  /**
   * {@inheritdoc}
   */
  public function spamMasterIsNeedle($spamFormId) {

    // Force needle straw.
    $spammaster_force_needle = [
      'commerce_checkout_flow_multistep',
    ];
    if (str_replace($spammaster_force_needle, '', $spamFormId) !== $spamFormId) {
      return "STRAW";
    }
    else {
      // Hide sig but force needle.
      $spammaster_hide_sig = [
        'quiz',
        'mailchimp',
      ];
      if (str_replace($spammaster_hide_sig, '', $spamFormId) !== $spamFormId) {
        return "ISHIDDEN";
      }
      else {
        // Exempt needle.
        $spammaster_exempt_needle = [
          'blockshop',
          'cart',
          'color_calc_form',
          'commerce_checkout_flow_multistep_default',
          'commerce_wishlist',
          'content_moderation_entity',
          'editor_image_dialog',
          'editor_link_dialog',
          'entity_view_display_layout_builder',
          'mailchimp',
          'mailchimp_signup_subscribe_block_sign_up_form',
          'media_library_add_form_oembed',
          'node_black_list_form',
          'node_verification_full_form',
          'profile_klubkartya_edit_form',
          'puppy_calc_form',
          'quiz',
          'rabbit_calc_form',
          'rate',
          'search',
          'simplenews_subscri',
          'shop',
          'tfa_entry_form',
          'views_',
        ];
        if (str_replace($spammaster_exempt_needle, '', $spamFormId) !== $spamFormId) {
          return "NEEDLE";
        }
        else {
          return "STRAW";
        }
      }
    }
  }

  /**
   * {@inheritdoc}
   */
  public function spamMasterFormData($form, $formstate, $spamFormId) {

    // Try to get the email.
    if (!empty($formstate->getValue('name')) && NULL !== $formstate->getValue('name') && !is_array($formstate->getValue('name'))) {
      if (filter_var($formstate->getValue('name'), FILTER_VALIDATE_EMAIL)) {
        $spammasteremail = $formstate->getValue('name');
      }
    }
    if (!empty($formstate->getValue('mail')) && NULL !== $formstate->getValue('mail')) {
      $spammasteremail = $formstate->getValue('mail');
    }
    if (!empty($formstate->getValue('field_email')) && NULL !== $formstate->getValue('field_email')) {
      if (is_array($formstate->getValue('field_email'))) {
        $spammastermessage = $formstate->getValue('field_email')[0]['value'];
      }
      else {
        $spammastermessage = $formstate->getValue('field_email');
      }
    }
    if (!empty($formstate->getValue('email')) && NULL !== $formstate->getValue('email')) {
      $spammasteremail = $formstate->getValue('email');
    }
    if (!empty($formstate->getValue('e_mail')) && NULL !== $formstate->getValue('e_mail')) {
      $spammasteremail = $formstate->getValue('e_mail');
    }
    if (!empty($formstate->getValue('your_email')) && NULL !== $formstate->getValue('your_email')) {
      $spammasteremail = $formstate->getValue('your_email');
    }
    if (empty($spammasteremail) || is_array($spammasteremail)) {
      $spammasteremail = 'drup@' . date('Ymdhis') . '.drup';
    }
    // Try to get the message.
    if (!empty($formstate->getValue('message')) && NULL !== $formstate->getValue('message')) {
      if (is_array($formstate->getValue('message'))) {
        $spammastermessage = $formstate->getValue('message')[0]['value'];
      }
      else {
        $spammastermessage = $formstate->getValue('message');
      }
    }
    if (!empty($formstate->getValue('bericht')) && NULL !== $formstate->getValue('bericht')) {
      if (is_array($formstate->getValue('bericht'))) {
        $spammastermessage = $formstate->getValue('bericht')[0]['value'];
      }
      else {
        $spammastermessage = $formstate->getValue('bericht');
      }
    }
    if (!empty($formstate->getValue('nachricht')) && NULL !== $formstate->getValue('nachricht')) {
      if (is_array($formstate->getValue('nachricht'))) {
        $spammastermessage = $formstate->getValue('nachricht')[0]['value'];
      }
      else {
        $spammastermessage = $formstate->getValue('nachricht');
      }
    }
    if (!empty($formstate->getValue('description')) && NULL !== $formstate->getValue('description')) {
      if (is_array($formstate->getValue('description'))) {
        $spammastermessage = $formstate->getValue('description')[0]['value'];
      }
      else {
        $spammastermessage = $formstate->getValue('description');
      }
    }
    if (!empty($formstate->getValue('comment_body')) && NULL !== $formstate->getValue('comment_body')) {
      if (is_array($formstate->getValue('comment_body'))) {
        $spammastermessage = $formstate->getValue('comment_body')[0]['value'];
      }
      else {
        $spammastermessage = $formstate->getValue('comment_body');
      }
    }
    if (!empty($formstate->getValue('field_message')) && NULL !== $formstate->getValue('field_message')) {
      if (is_array($formstate->getValue('field_message'))) {
        $spammastermessage = $formstate->getValue('field_message')[0]['value'];
      }
      else {
        $spammastermessage = $formstate->getValue('field_message');
      }
    }
    if (!empty($formstate->getValue('additional_information')) && NULL !== $formstate->getValue('additional_information')) {
      if (is_array($formstate->getValue('additional_information'))) {
        $spammastermessage = $formstate->getValue('additional_information')[0]['value'];
      }
      else {
        $spammastermessage = $formstate->getValue('additional_information');
      }
    }
    if (empty($spammastermessage)) {
      // If still no message try to replace with name.
      if (!empty($formstate->getValue('name')) && NULL !== $formstate->getValue('name')) {
        $spammastermessage = 'Content: ' . $formstate->getValue('name');
      }
      else {
        $spammastermessage = 'contentless';
      }
    }

    $prespamformdata = [
      'spammasteremail'   => $spammasteremail,
      'spammastermessage' => $spammastermessage,
    ];
    $spamformdata = Json::encode($prespamformdata);

    return $spamformdata;

  }

}
